from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class EFSEncryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure EFS is securely encrypted"
        id = "CKV_AWS_42"
        supported_resources = ['AWS::EFS::FileSystem']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
        Looks for encryption configuration at aws_efs_filesystem:
        https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-efs-filesystem.html
        :param conf: aws_efs_filesystem configuration
        :return: <CheckResult>
        """
        if conf.get('Properties'):
            if conf['Properties'].get('Encrypted'):
                return CheckResult.PASSED
        return CheckResult.FAILED


check = EFSEncryption()



from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class GKEHasLabels(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Kubernetes Clusters are configured with Labels"
        id = "CKV_GCP_21"
        supported_resources = ['google_container_cluster']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for password configuration at azure_instance:
            https://www.terraform.io/docs/providers/google/r/compute_ssl_policy.html
        :param conf: google_compute_ssl_policy configuration
        :return: <CheckResult>
        """
        if 'resource_labels' in conf:
            resource_labels = conf.get('resource_labels')
            if isinstance(resource_labels[0], dict) and len(resource_labels[0].keys()) > 0:
                return CheckResult.PASSED
        return CheckResult.FAILED


check = GKEHasLabels()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_list
from checkov.common.util.type_forcers import force_int


class AbsSecurityGroupUnrestrictedIngress(BaseResourceCheck):
    def __init__(self, check_id, port):
        name = "Ensure no security groups allow ingress from 0.0.0.0:0 to port %d" % port
        supported_resources = ['aws_security_group']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=check_id, categories=categories, supported_resources=supported_resources)
        self.port = port

    def scan_resource_conf(self, conf):
        """
            Looks for configuration at security group ingress rules :
            https://www.terraform.io/docs/providers/aws/r/security_group.html
        :param conf: aws_security_group configuration
        :return: <CheckResult>
        """
        if 'ingress' in conf:
            ingress_conf = conf['ingress']
            for ingress_rule in ingress_conf:
                ingress_rules = force_list(ingress_rule)
                for rule in ingress_rules:
                    if isinstance(rule, dict):
                        from_port = force_int(force_list(rule['from_port'])[0])
                        to_port = force_int(force_list(rule['to_port'])[0])

                        if from_port is not None and to_port is not None and from_port <= self.port <= to_port:
                            # It's not clear whether these can ever be a type other
                            # than an empty list but just in case…
                            cidr_blocks = force_list(rule.get('cidr_blocks', [[]])[0])
                            security_groups = rule.get('security_groups', [])

                            if "0.0.0.0/0" in cidr_blocks and not security_groups:
                                return CheckResult.FAILED

        return CheckResult.PASSED



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.base_spec_check import BaseK8Check


class ImageDigest(BaseK8Check):

    def __init__(self):
        """
        The image specification should use a digest instead of a tag to make sure the container always uses the same
        version of the image.
        https://kubernetes.io/docs/concepts/configuration/overview/#container-images

        An admission controller could be used to enforce the use of image digest
        """
        name = "Image should use digest"
        id = "CKV_K8S_43"
        # Location: container .image
        supported_kind = ['containers', 'initContainers']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def get_resource_id(self, conf):
        return conf['parent']

    def scan_spec_conf(self, conf):
        if "image" in conf:

            # The @ indicates there is a digest. It's technically possible to use a tag as well, but it doesn't make
            # a difference. So, this @ is all we need to pass the check.
            image_conf = conf["image"]
            if isinstance(image_conf,str):
                has_digest = '@' in image_conf
                return CheckResult.PASSED if has_digest else CheckResult.FAILED
        else:
            return CheckResult.FAILED


check = ImageDigest()


from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck


class PostgreSQLServerConnectionThrottlingEnabled(BaseResourceCheck):
    def __init__(self):
        name = "Ensure server parameter 'connection_throttling' is set to 'ON' for PostgreSQL Database Server"
        id = "CKV_AZURE_32"
        supported_resources = ['azurerm_postgresql_configuration']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if conf['name'][0] == 'connection_throttling' and conf['value'][0] == 'off':
            return CheckResult.FAILED
        return CheckResult.PASSED


check = PostgreSQLServerConnectionThrottlingEnabled()
